package com.gic.haoban.manage.service.service.out.impl.fee;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.constants.Manage3Constants;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.fee.HaobanQywxFeeAccountInterceptLogDTO;
import com.gic.haoban.manage.api.dto.fee.HaobanQywxFeeDTO;
import com.gic.haoban.manage.api.dto.fee.HaobanQywxFeeDetailDTO;
import com.gic.haoban.manage.api.dto.qdto.QywxFeeListQDTO;
import com.gic.haoban.manage.api.service.fee.HaobanQywxFeeApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.entity.fee.*;
import com.gic.haoban.manage.service.pojo.bo.fee.HaobanQywxFeeBO;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.fee.*;
import com.gic.wechat.api.dto.qywx.fee.*;
import com.gic.wechat.api.dto.qywx.fee.qdto.FeeOrderListQDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("haobanQywxFeeApiService")
public class HaobanQywxFeeApiServiceImpl implements HaobanQywxFeeApiService {

    private static Logger logger = LoggerFactory.getLogger(HaobanQywxFeeApiServiceImpl.class);

    @Autowired
    private HaobanQywxFeeService haobanQywxFeeService;
    @Autowired
    private HaobanQywxFeeOrderService haobanQywxFeeOrderService;
    @Autowired
    private HaobanQywxFeeOrderAccountService haobanQywxFeeOrderAccountService;
    @Autowired
    private HaobanQywxFeeAccountStaffService haobanQywxFeeAccountStaffService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private Config config;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private HaobanQywxFeeAccountInterceptLogService haobanQywxFeeAccountInterceptLogService;

    @Override
    public void insert(String param) {
        List<WxEnterpriseDTO> wxEnterpriseDTOS = wxEnterpriseService.listAllEnterprise();
        String serviceCorpid = config.getCorpid();
        for (WxEnterpriseDTO wxEnterpriseDTO : wxEnterpriseDTOS) {
            String corpid = wxEnterpriseDTO.getCorpid();
            String wxEnterpriseId = wxEnterpriseDTO.getWxEnterpriseId();
            String autoActiveStatus = qywxUserApiService.getAutoActiveStatus(corpid, serviceCorpid);
            if (!"1".equals(autoActiveStatus)) {
                logger.info("未开启自动激活：{}", corpid);
                wxEnterpriseService.updateAutoActiveFlagById(0, wxEnterpriseId);
                continue;
            }
            logger.info("企微收费：{},{}", corpid, wxEnterpriseId);

            //更新订单
            List<String> orderIds = new ArrayList<>();
            while (true) {
                FeeOrderListQDTO feeOrderListQDTO = new FeeOrderListQDTO();
                feeOrderListQDTO.setCorpid(corpid);
                feeOrderListQDTO.setLimit(Manage3Constants.QW_LIMIT);
                FeeOrderResponseList feeOrderResponseList = qywxUserApiService.listOrder(corpid, serviceCorpid, feeOrderListQDTO);
                int errcode = feeOrderResponseList.getErrcode();
                if (errcode != 0) {
                    logger.error("查询订单列表失败：{}", feeOrderResponseList.getErrmsg());
                    break;
                }
                for (FeeOrderResponseList.OrderListDTO orderListDTO : feeOrderResponseList.getOrderList()) {
                    orderIds.add(orderListDTO.getOrderId());
                }
                Integer hasMore = feeOrderResponseList.getHasMore();
                if (hasMore == 0) {
                    logger.error("订单无更多账号");
                    break;
                }
                feeOrderListQDTO.setCursor(feeOrderResponseList.getNextCursor());
                feeOrderResponseList = qywxUserApiService.listOrder(corpid, serviceCorpid, feeOrderListQDTO);
                for (FeeOrderResponseList.OrderListDTO orderListDTO : feeOrderResponseList.getOrderList()) {
                    orderIds.add(orderListDTO.getOrderId());
                }
            }

            List<String> activeCodes = new ArrayList<>();
            for (String orderId : orderIds) {
                FeeOrderDeatilResponseDTO order = qywxUserApiService.getOrder(corpid, serviceCorpid, orderId);
                if (order.getErrcode() != 0) {
                    logger.error("请求企微失败：{}", orderId);
                    break;
                }

                //插入订单
                FeeOrderDeatilResponseDTO.OrderDTO orderDTO = order.getOrder();
                Integer orderStatus = orderDTO.getOrderStatus();
                if (orderStatus != 1) {
                    logger.info("只处理已支付订单:{}", orderId);
                    continue;
                }

                TabHaobanQywxFeeOrder tabHaobanQywxFeeOrder = new TabHaobanQywxFeeOrder();
                tabHaobanQywxFeeOrder.setFeeOrderId(UniqueIdUtils.uniqueLong());
                tabHaobanQywxFeeOrder.setOrderId(orderId);
                tabHaobanQywxFeeOrder.setCorpId(corpid);
                tabHaobanQywxFeeOrder.setWxEnterpriseId(wxEnterpriseId);
                tabHaobanQywxFeeOrder.setOrderType(orderDTO.getOrderType());
                tabHaobanQywxFeeOrder.setOrderStatus(orderStatus);
                tabHaobanQywxFeeOrder.setPrice(new BigDecimal(orderDTO.getPrice()));
                tabHaobanQywxFeeOrder.setBaseCount(orderDTO.getAccountCount().getBaseCount());
                tabHaobanQywxFeeOrder.setExternalContactCount(orderDTO.getAccountCount().getExternalContactCount());
                tabHaobanQywxFeeOrder.setMonths(orderDTO.getAccountDuration().getMonths());
                tabHaobanQywxFeeOrder.setCreateTime(DateUtil.date(orderDTO.getCreateTime()));
                tabHaobanQywxFeeOrder.setPayTime(DateUtil.date(orderDTO.getPayTime()));
                haobanQywxFeeOrderService.insert(tabHaobanQywxFeeOrder);

            }

            //更新账号
            while (true) {
                FeeOrderListQDTO feeOrderListQDTO = new FeeOrderListQDTO();
                feeOrderListQDTO.setCorpid(corpid);
                feeOrderListQDTO.setLimit(Manage3Constants.QW_LIMIT);
                AccountListResponseDTO accountListResponseDTO = qywxUserApiService.listActivedAccount(corpid, serviceCorpid, feeOrderListQDTO);

                if (accountListResponseDTO.getErrcode() != 0) {
                    logger.info("订单账号列表报错：{}", accountListResponseDTO.getErrmsg());
                    break;
                }

                List<AccountListDTO> accountList = accountListResponseDTO.getAccountList();
                for (AccountListDTO accountListDTO : accountList) {
                    activeCodes.add(accountListDTO.getActiveCode());
                }

                Integer hasMore = accountListResponseDTO.getHasMore();
                if (hasMore == 0) {
                    logger.error("无更多账号");
                    break;
                }
                feeOrderListQDTO.setCursor(feeOrderListQDTO.getCursor());
                accountListResponseDTO = qywxUserApiService.listActivedAccount(corpid, serviceCorpid, feeOrderListQDTO);
                for (AccountListDTO accountListDTO : accountListResponseDTO.getAccountList()) {
                    activeCodes.add(accountListDTO.getActiveCode());
                }
            }

            for (String activeCode : activeCodes) {
                ActiveInfoCodeResponseDTO activeInfoCodeResponseDTO = qywxUserApiService.getActiveInfoByCode(corpid, serviceCorpid, activeCode);
                if (activeInfoCodeResponseDTO.getErrcode() != 0) {
                    logger.error("激活码详情报错：{}", activeInfoCodeResponseDTO.getErrmsg());
                    continue;
                }
                String userid = activeInfoCodeResponseDTO.getUserid();
                TabHaobanQywxFeeOrderAccount tabHaobanQywxFeeOrderAccount = new TabHaobanQywxFeeOrderAccount();
                tabHaobanQywxFeeOrderAccount.setFeeAccountId(UniqueIdUtils.uniqueLong());
                //tabHaobanQywxFeeOrderAccount.setOrderId();
                tabHaobanQywxFeeOrderAccount.setCorpId(corpid);
                tabHaobanQywxFeeOrderAccount.setWxEnterpriseId(wxEnterpriseId);
                tabHaobanQywxFeeOrderAccount.setWxUserId(userid);
                tabHaobanQywxFeeOrderAccount.setActiveCode(activeInfoCodeResponseDTO.getActiveCode());
                tabHaobanQywxFeeOrderAccount.setAccountType(activeInfoCodeResponseDTO.getType());
                tabHaobanQywxFeeOrderAccount.setStatus(activeInfoCodeResponseDTO.getStatus());
                tabHaobanQywxFeeOrderAccount.setExpireTime(DateUtil.date(activeInfoCodeResponseDTO.getExpireTime()));
                tabHaobanQywxFeeOrderAccount.setActiveTime(DateUtil.date(activeInfoCodeResponseDTO.getActiveTime()));
                haobanQywxFeeOrderAccountService.insert(tabHaobanQywxFeeOrderAccount);

                //更新成员
                ActiveInfoUserResponseDTO activeInfoUserResponseDTO = qywxUserApiService.getActiveInfoByUser(corpid, serviceCorpid, userid);
                if (activeInfoUserResponseDTO.getErrcode() != 0) {
                    logger.info("成员激活详情报错：{}", activeInfoUserResponseDTO.getErrmsg());
                    break;
                }
                List<AccountListDTO> activeInfoList = activeInfoUserResponseDTO.getActiveInfoList();
                Integer activeStatus = activeInfoUserResponseDTO.getActiveStatus();
                TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(userid, wxEnterpriseId);
                for (AccountListDTO accountListDTO : activeInfoList) {
                    TabHaobanQywxFeeAccountStaff tabHaobanQywxFeeAccountStaff = new TabHaobanQywxFeeAccountStaff();
                    tabHaobanQywxFeeAccountStaff.setFeeAccountId(UniqueIdUtils.uniqueLong());
                    //tabHaobanQywxFeeAccountStaff.setOrderId();
                    tabHaobanQywxFeeAccountStaff.setCorpId(corpid);
                    tabHaobanQywxFeeAccountStaff.setWxEnterpriseId(wxEnterpriseId);
                    tabHaobanQywxFeeAccountStaff.setWxUserId(userid);
                    if (tabHaobanStaff != null) {
                        String staffId = tabHaobanStaff.getStaffId();
                        tabHaobanQywxFeeAccountStaff.setStaffId(staffId);
                    }
                    tabHaobanQywxFeeAccountStaff.setActiveCode(accountListDTO.getActiveCode());
                    tabHaobanQywxFeeAccountStaff.setAccountType(accountListDTO.getType());
                    tabHaobanQywxFeeAccountStaff.setExpireTime(DateUtil.date(accountListDTO.getExpireTime()));
                    tabHaobanQywxFeeAccountStaff.setActiveTime(DateUtil.date(accountListDTO.getActiveTime()));
                    haobanQywxFeeAccountStaffService.insert(tabHaobanQywxFeeAccountStaff);
                }
                if (tabHaobanStaff != null) {
                    String staffId = tabHaobanStaff.getStaffId();
                    staffService.updateActiveStatusById(activeStatus, staffId);
                }
            }

            //更新企微状态
            wxEnterpriseService.updateAutoActiveFlagById(1, wxEnterpriseId);

            //更新收费
            HaobanQywxFeeDetailDTO haobanQywxFeeDetailDTO = haobanQywxFeeAccountStaffService.feeIntercept(wxEnterpriseId);
            if (haobanQywxFeeDetailDTO != null) {
                TabHaobanQywxFee tabHaobanQywxFee = new TabHaobanQywxFee();
                tabHaobanQywxFee.setQwFeeId(UniqueIdUtils.uniqueLong());
                tabHaobanQywxFee.setCorpName(wxEnterpriseDTO.getCorpName());
                tabHaobanQywxFee.setCorpId(corpid);
                tabHaobanQywxFee.setWxEnterpriseId(wxEnterpriseId);
                tabHaobanQywxFee.setInterceptTime(haobanQywxFeeDetailDTO.getInterceptTime());
                tabHaobanQywxFee.setInterceptUserNum(haobanQywxFeeDetailDTO.getInterceptUserNum());
                tabHaobanQywxFee.setPermissionUserNum(haobanQywxFeeOrderAccountService.countOrderAccountStatus(1,wxEnterpriseId));
                tabHaobanQywxFee.setBuyPermissionUserNum(haobanQywxFeeOrderService.countExternalContactCount(wxEnterpriseId));
                tabHaobanQywxFee.setInterceptFlag(0);
                haobanQywxFeeService.insert(tabHaobanQywxFee);
            }

            List<TabHaobanQywxFeeAccountInterceptLog> tabHaobanQywxFeeAccountInterceptLogs = haobanQywxFeeAccountInterceptLogService.queryByWxEnterpriseId(wxEnterpriseId);
            if (CollectionUtils.isNotEmpty(tabHaobanQywxFeeAccountInterceptLogs)) {
                TabHaobanQywxFee tabHaobanQywxFee = new TabHaobanQywxFee();
                tabHaobanQywxFee.setQwFeeId(UniqueIdUtils.uniqueLong());
                tabHaobanQywxFee.setCorpName(wxEnterpriseDTO.getCorpName());
                tabHaobanQywxFee.setCorpId(corpid);
                tabHaobanQywxFee.setWxEnterpriseId(wxEnterpriseId);
                tabHaobanQywxFee.setInterceptTime(DateUtil.offsetDay(new Date(),-1));
                tabHaobanQywxFee.setInterceptUserNum(tabHaobanQywxFeeAccountInterceptLogs.size());
                tabHaobanQywxFee.setPermissionUserNum(haobanQywxFeeOrderAccountService.countOrderAccountStatus(1,wxEnterpriseId));
                tabHaobanQywxFee.setBuyPermissionUserNum(haobanQywxFeeOrderService.countExternalContactCount(wxEnterpriseId));
                tabHaobanQywxFee.setInterceptFlag(1);
                haobanQywxFeeService.insert(tabHaobanQywxFee);
            }
        }
    }


    @Override
    public Page<HaobanQywxFeeDTO> listPage(QywxFeeListQDTO qywxFeeListQDTO) {
        PageHelper.startPage(qywxFeeListQDTO.getPageNum(),qywxFeeListQDTO.getPageSize());
        List<HaobanQywxFeeBO> haobanQywxFeeBOS = haobanQywxFeeService.listPage(qywxFeeListQDTO);
        return PageUtil.changePageHelperToCurrentPage(new PageInfo<>(haobanQywxFeeBOS), HaobanQywxFeeDTO.class);
    }

    @Override
    public List<HaobanQywxFeeDetailDTO> feeDetail(String wxEnterpriseId) {
        return haobanQywxFeeAccountStaffService.feeDetail(wxEnterpriseId);
    }

    @Override
    public void addInterceptLog(String param) {
        HaobanQywxFeeAccountInterceptLogDTO haobanQywxFeeAccountInterceptLogDTO = JSON.toJavaObject(JSON.parseObject(param),HaobanQywxFeeAccountInterceptLogDTO.class);
        haobanQywxFeeAccountInterceptLogDTO.setFeeAccountInterceptId(UniqueIdUtils.uniqueLong());
        TabHaobanWxEnterprise haobanWxEnterprise = wxEnterpriseService.getEnterpriseBycorpId(haobanQywxFeeAccountInterceptLogDTO.getCorpId());
        if (haobanWxEnterprise != null) {
            String wxEnterpriseId = haobanWxEnterprise.getWxEnterpriseId();
            haobanQywxFeeAccountInterceptLogDTO.setWxEnterpriseId(wxEnterpriseId);
            TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(haobanQywxFeeAccountInterceptLogDTO.getWxUserId(), wxEnterpriseId);
            if (tabHaobanStaff != null) {
                haobanQywxFeeAccountInterceptLogDTO.setStaffId(tabHaobanStaff.getStaffId());
            }
        }
        haobanQywxFeeAccountInterceptLogService.insert(EntityUtil.changeEntityNew(TabHaobanQywxFeeAccountInterceptLog.class,haobanQywxFeeAccountInterceptLogDTO));
    }
}
